/*
Monitor calibration shader by Simon Zajac, 2019.
Used by Calibration Tools to apply monitor calibration with high precision dithering for smooth gradients.
*/

NAMESPACE_ENTER(CFX)
NAMESPACE_ENTER(CALIBRATION_TOOLS)
#include CFX_SETTINGS_DEF
#if (USE_CALIBRATIONTOOLS == 1)
	


//define the texture LUT used for holding the video card gamma table
#if (CT_ApplyCal == 1)	
texture texLUT < string source = "ReShade/CustomFX/Textures/CT_vcgt.png"; > {Width = 3; Height = 256; Format = RGBA8;};
sampler2D SamplerLUT
{		  
	Texture = texLUT;
						  // update 2024-01-31:
	//MinFilter = NONE;   "NONE" only avoids banding if the backbuffer's RGB values are aligned to 1/255 since the texLUT 
	//MagFilter = NONE;   uses 1/255 alignment for the lookup.  If backbuffer isn't aligned to 1/255, eg. if previous effects
	//MipFilter = NONE;   in the pipeline have been enabled, banding will occur.  Therefore use linear interpolation instead:
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	
	AddressU = Clamp;
	AddressV = Clamp; 

};
#endif



//timer used for temporal dithering
#if (CT_ApplyDither == 2)  //2 = temporal dithering
uniform float timer < source = "timer"; >;
#endif



//main function for the pixel shader
float4 PS_CalibrationTools(float4 vpos : SV_Position, float2 texcoord : TEXCOORD) : SV_Target
{
	//get pixel colour from back buffer	
	float4 col = tex2D(RFX_backbufferColor, texcoord);	
	
	//get steps of gradation used by the game
	float gradationSteps; gradationSteps = pow(2.0,float(CT_BitDepth)) - 1.0;
	
	
	#if (CT_ApplyCal == 1)	
	//look up colour from the 3x256 texture LUT
	col.r = saturate
			(
				   (tex2D(SamplerLUT, float2(0.0f,saturate(col.r))).r)	 //(1/255) doesn't work
				+ ((tex2D(SamplerLUT, float2(0.5f,saturate(col.r))).r) * (0.0039215686274509f)) 
				+ ((tex2D(SamplerLUT, float2(1.0f,saturate(col.r))).r) * (0.0000153787004998f))
			);
		
	col.g = saturate
			(		
				 (tex2D(SamplerLUT, float2(0.0f,saturate(col.g))).g)
			  + ((tex2D(SamplerLUT, float2(0.5f,saturate(col.g))).g) * (0.0039215686274509f))
			  + ((tex2D(SamplerLUT, float2(1.0f,saturate(col.g))).g) * (0.0000153787004998f))
			);
		
	col.b = saturate
			(
				 (tex2D(SamplerLUT, float2(0.0f,saturate(col.b))).b)
			  + ((tex2D(SamplerLUT, float2(0.5f,saturate(col.b))).b) * (0.0039215686274509f))
			  + ((tex2D(SamplerLUT, float2(1.0f,saturate(col.b))).b) * (0.0000153787004998f))
			);			
	#endif
		
	
	#if (CT_ApplyDither == 1) //1 = static dithering 	
	float seed = dot(texcoord, float2(12.9898,78.233));
	float noise = frac(frac(sin(seed)) * 43758.5453);
	float ditherStrength = CT_DitherStrength / gradationSteps;
	noise = (noise * ditherStrength) - (ditherStrength * 0.5);
	#endif
	
	
	#if (CT_ApplyDither == 2) //2 = temporal dithering 
	float seed = dot(texcoord, float2(12.9898,78.233)) + frac(timer/1000);
	float noise = frac(frac(sin(seed)) * 43758.5453);
	float ditherStrength = CT_DitherStrength / gradationSteps;
	noise = (noise * ditherStrength) - (ditherStrength * 0.5);
	#endif
	

	#if (CT_ApplyDither == 1) || (CT_ApplyDither == 2)	
	
		#if (CT_DitherSubpixel == 1) 
		//subpixel dithering: shift green-magenta by noise value	
		if ((col.r != 0) && (col.r != 1)) //avoid raising black or reducing white
			col.r += -noise;
		if ((col.g != 0) && (col.g != 1))
			col.g += noise;
		if ((col.b != 0) && (col.b != 1))
			col.b += -noise;
		#endif
		
		#if (CT_DitherSubpixel == 0) 
		if ((col.r != 0) && (col.r != 1)) //avoid raising black or reducing white
			col.r += noise;
		if ((col.g != 0) && (col.g != 1))
			col.g += noise;
		if ((col.b != 0) && (col.b != 1))
			col.b += noise;
		#endif
				
	#endif
		
			
	//manually round to nearest gradation step, else slightly incorrect values according to FRAFS	
	col.rgb = saturate(round(col.rgb * gradationSteps)/gradationSteps); 
	return col;		
}



technique Monitor_Calibration_Shader <bool enabled = RFX_Start_Enabled; int toggle = CalibrationTools_ToggleKey; >
{
	pass CalibrationToolsPass
	{
		VertexShader = RFX_VS_PostProcess;
		PixelShader = PS_CalibrationTools;
	}
}



#endif
#include CFX_SETTINGS_UNDEF
NAMESPACE_LEAVE()
NAMESPACE_LEAVE()